/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.dao.filesystem;

import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import br.registro.dnsshim.common.server.DnsshimProtocolException;
import br.registro.dnsshim.domain.Zone;
import br.registro.dnsshim.domain.ZoneIncrement;
import br.registro.dnsshim.util.DomainNameUtil;
import br.registro.dnsshim.xfrd.domain.ZoneDataPrePublish;
import br.registro.dnsshim.xfrd.domain.Zonename;
import br.registro.dnsshim.xfrd.repository.ZoneDataPrePublishRepository;
import br.registro.dnsshim.xfrd.repository.ZoneIncrementRepository;
import br.registro.dnsshim.xfrd.repository.ZoneRepository;
import br.registro.dnsshim.xfrd.repository.ZonenameRepository;

public class ZoneDataPrePublishDao implements ZoneDataPrePublishRepository {
	private static final Logger logger = Logger.getLogger(ZoneDataPrePublishDao.class);
	private ZoneRepository zoneRepository = new ZonePrePublishDao();
	private ZoneIncrementRepository zoneIncrementRepository = new ZoneIncrementPrePublishDao(); 


	@Override
	public void save(ZoneDataPrePublish zonePrePublish) throws IOException {
		if (logger.isDebugEnabled()) {
			logger.debug("Saving ZoneDataPrePublish " + zonePrePublish.getZonename());
		}
		Zone axfr = zonePrePublish.getAxfr();
		zoneRepository.save(axfr);
		
		ZoneIncrement ixfr = zonePrePublish.getIxfr();
		zoneIncrementRepository.save(ixfr);	
	}
	
	public void delete(String zonename) throws IOException {
		zoneRepository.delete(zonename);
	}

	@Override
	public ZoneDataPrePublish findByZonename(String zonename)
		throws IOException, DnsshimProtocolException {
		String zn = DomainNameUtil.trimDot(zonename);
		if (logger.isDebugEnabled()) {
			logger.debug("Finding " + zn + " ZoneData");
		}
		
		ZoneDataPrePublish zonePrePublish;
		
		Zone axfr = zoneRepository.findByZonename(zonename);
		if (axfr == null) {
			return null;
		}
		
		Map<Long, ZoneIncrement> ixfrs = zoneIncrementRepository.findByZonename(zonename);
		if (!ixfrs.isEmpty()) {
			ZoneIncrement ixfr = ixfrs.values().iterator().next();
			zonePrePublish = new ZoneDataPrePublish(axfr, ixfr);
		} else {
			zonePrePublish = new ZoneDataPrePublish(axfr);
		}
	
		return zonePrePublish;
	}

	@Override
	public Map<String, ZoneDataPrePublish> findAll()
		throws IOException, DnsshimProtocolException {
		Map<String, ZoneDataPrePublish> listZonePrePub = new LinkedHashMap<String, ZoneDataPrePublish>();
		ZonenameRepository repository = new ZonenameDao();
		
		for (Zonename zonename : repository.findAll()) {
			ZoneDataPrePublish zonePrePublish;
			Zone axfr = zoneRepository.findByZonename(zonename.getName());
			Map<Long, ZoneIncrement> ixfrs = zoneIncrementRepository.findByZonename(zonename.getName());
			if (!ixfrs.isEmpty()) {
				ZoneIncrement ixfr = (ZoneIncrement) ixfrs.values().toArray()[0];
				zonePrePublish = new ZoneDataPrePublish(axfr, ixfr);
			} else {
				zonePrePublish = new ZoneDataPrePublish(axfr);
			}
			listZonePrePub.put(zonename.getName(), zonePrePublish);
		}
		
		return listZonePrePub;
	}
}