/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.domain;

import java.util.Map;
import java.util.TreeMap;

import br.registro.dnsshim.domain.logic.Cacheable;

public class TsigKeyInfo implements Cacheable<String> {
	private String host;
	private Map<String, String> keys;
	
	public TsigKeyInfo() {
		keys = new TreeMap<String, String>();
	}
	
	@Override
	public String getCacheKey() {
		return this.host;
	}

	public String getHost() {
		return host;
	}

	public void setHost(String host) {
		this.host = host;
	}

	public Map<String, String> getKeys() {
		return keys;
	}

	public void setKeys(Map<String, String> keys) {
		this.keys = keys;
	}
	
	public boolean contains(String key) {
		if (keys.keySet().contains(key)) {
			return true;
		}
		
		return false;
	}
	
	public String getKey(String key) {
		if (!contains(key))	 {
			return "";
		}
		
		return keys.get(key);
	}
	
	public void add(String key, String secret) {
		if (contains(key)) {
			return;
		}
		
		keys.put(key, secret);
	}
	
	public boolean remove(String key) {
		if (!contains(key)) {
			return false;
		}
		
		keys.remove(key);
		return true;
	}
	
	public boolean isEmpty() {
		return this.keys.isEmpty();
	}
	
	@Override
	public String toString() {
		StringBuffer result = new StringBuffer();
		result.append("host = ").append(host).append(" ");
		result.append("keys [");
		for (String key : keys.keySet()) {
			result.append(key).append(": ").append(keys.get(key)).append(" ");
		}
		result.append(']');
		
		return result.toString();
	}
	
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof TsigKeyInfo) {
			TsigKeyInfo tsigKeyInfo = (TsigKeyInfo) obj;
			if (this.host.equals(tsigKeyInfo.host) &&
					this.keys.equals(tsigKeyInfo.keys)) {
				return true;
			}
		}
		return false;
	}
}
